@@ -1,4 +1,4 @@
-# Copyright 2004-2021 Bright Computing Holding BV
+# Copyright 2004-2022 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,7 +36,12 @@ from six.moves.mock import Mock, PropertyMock, patch
 
 from clusterondemand import const
 from clusterondemand.command_runner import sys_argv
-from clusterondemandconfig import BCM_VERSION, global_configuration, load_configuration_for_command
+from clusterondemandconfig import (
+    BCM_VERSION,
+    global_configuration,
+    load_configuration_for_command,
+    validate_configuration
+)
 from clusterondemandconfig.determine_invoked_command import determine_invoked_command
 
 
@@ -67,6 +72,84 @@ def ipxe_plain_eth0():
     return m
 
 
+def default_boot_image():
+    m = Mock(
+        spec="",
+        status=u"active",
+        tags=[],
+        container_format=u"bare",
+        min_ram=0,
+        updated_at=u"2018-03-17T21:46:33",
+        visibility=u"public",
+        owner=u"3b9bd8793720479f8bb341edc808a499",
+        file=u"/v2/images/bba054ed-3ac7-4fcd-9401-ca233cae2eba/file",
+        min_disk=0,
+        virtual_size=None,
+        id=u"bba054ed-3ac7-4fcd-9401-ca233cae2eba",
+        size=8388608,
+        checksum=u"3661bd33484e39c4e9d75d15fa61b0ea",
+        created_at=u"2018-03-17T21:46:32",
+        disk_format=u"raw",
+        protected=False,
+        direct_url=u"rbd://yadayada",
+        schema=u"/v2/schemas/image"
+    )
+    type(m).name = PropertyMock(return_value="default-boot-image")
+    return m
+
+
+def custom_boot_image():
+    m = Mock(
+        spec="",
+        status=u"active",
+        tags=[],
+        container_format=u"bare",
+        min_ram=0,
+        updated_at=u"2018-03-17T21:46:33",
+        visibility=u"public",
+        owner=u"3b9bd8793720479f8bb341edc808a499",
+        file=u"/v2/images/bba054ed-3ac7-4fcd-9401-ca115cae2bbb/file",
+        min_disk=0,
+        virtual_size=None,
+        id=u"bba054ed-3ac7-4fcd-9401-ca115cae2bbb",
+        size=8388608,
+        checksum=u"3661bd33484e39c4e9d75d15fa61b0ea",
+        created_at=u"2018-03-17T21:46:32",
+        disk_format=u"raw",
+        protected=False,
+        direct_url=u"rbd://yadayada",
+        schema=u"/v2/schemas/image"
+    )
+    type(m).name = PropertyMock(return_value="custom-boot-image")
+    return m
+
+
+def flavor_boot_image():
+    m = Mock(
+        spec="",
+        status=u"active",
+        tags=[],
+        container_format=u"bare",
+        min_ram=0,
+        updated_at=u"2018-03-17T21:46:33",
+        visibility=u"public",
+        owner=u"3b9bd8793720479f8bb341edc808a499",
+        file=u"/v2/images/bba054ed-3ac7-4fcd-9401-ca115cae2baa/file",
+        min_disk=0,
+        virtual_size=None,
+        id=u"bba054ed-3ac7-4fcd-9401-ca115cae2baa",
+        size=8388608,
+        checksum=u"3661bd33484e39c4e9d75d15fa61b0ea",
+        created_at=u"2018-03-17T21:46:32",
+        disk_format=u"raw",
+        protected=False,
+        direct_url=u"rbd://yadayada",
+        schema=u"/v2/schemas/image"
+    )
+    type(m).name = PropertyMock(return_value="flavor-boot-image")
+    return m
+
+
 def bcmh_centos7u4():
     m = Mock(
         spec="",
@@ -93,9 +176,7 @@ def bcmh_centos7u4():
         size=14289993728,
         bcm_cmd_revision=u"132465",
         bcm_image_id=f"centos7u4-{BCM_VERSION}",
-        bcm_cmd_hash=u"af65b9f1435",
-        bcm_cluster_tools_hash=u"ce3b53adad",
-        bcm_cm_setup_hash=u"67557b1d0d2",
+        bcm_pkg_hashes="cmdaemon=af65b9f1435,cm-setup=67557b1d0d2,cluster-tools=ce3b53adad",
         bcm_cloud_type=u"openstack",
         bcm_created_at=u"2018-04-08T23:41:06",
         created_at=u"2018-04-08T23:41:06",
@@ -188,6 +269,16 @@ FLAVOR_COD_XSMALL = Mock(get_keys=Mock(return_value={}),
                          id="a39c2730-6988-11e8-adc0-fa7ae01bbebc")
 type(FLAVOR_COD_XSMALL).name = PropertyMock(return_value="cod.xsmall")
 
+FLAVOR_COD_CUSTOM_BOOT = Mock(get_keys=Mock(return_value={
+                              "node_boot_image": "flavor-boot-image"
+                              }),
+                              ram=1024,
+                              vcpus=1,
+                              disk=0,
+                              ephemeral=0,
+                              id="a39c2730-6999-11e8-adc0-fa7ae01bbebc")
+type(FLAVOR_COD_CUSTOM_BOOT).name = PropertyMock(return_value="cod.custom.boot")
+
 TESTNET = {
     "status": "ACTIVE",
     "id": "660e44f0-5037-4231-b9ff-35009d33b041",
@@ -321,7 +412,8 @@ def patch_clientutils(func):
                 get_heat_client, get_cinder_client, *args, **kwargs):
         def return_images_list(filters=None, page_size=25):
             filters = {} if filters is None else filters
-            images_list = [ipxe_plain_eth0(), ipxe_plain_eth1(), bcmh_centos7u4(), ipxe_caas()]
+            images_list = [ipxe_plain_eth0(), default_boot_image(), custom_boot_image(), flavor_boot_image(),
+                           ipxe_plain_eth1(), bcmh_centos7u4(), ipxe_caas()]
 
             def filter_func(image):
                 for k, v in six.iteritems(filters):
@@ -344,7 +436,7 @@ def patch_clientutils(func):
 
         def return_flavors_list():
             flavors = [FLAVOR_M1_XSMALL, FLAVOR_M1_MEDIUM, FLAVOR_BAREMETAL,
-                       FLAVOR_COD_MEDIUM, FLAVOR_COD_XSMALL]
+                       FLAVOR_COD_MEDIUM, FLAVOR_COD_XSMALL, FLAVOR_COD_CUSTOM_BOOT]
             return flavors
 
         def return_volume_types():
@@ -435,12 +527,16 @@ class environ(object):
         return wrapper
 
 
-def run_command(cli, command_context):
+def run_command(cli, command_context, validate=False):
     with sys_argv(cli):
         command, clean_sys_argv = determine_invoked_command(command_context)
 
     with sys_argv(clean_sys_argv):
         config = load_configuration_for_command(command, system_config_files=[], enforcing_config_files=[])
+
+    if validate:
+        validate_configuration(config)
+
     with global_configuration(config):
         command.run_command()
 
